<?php

namespace FiloBlu\AdyenExchangeRate\Model;

use FiloBlu\AdyenExchangeRate\Model\ExchangeRateRepository;
use FiloBlu\AdyenExchangeRate\Model\Retriever;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Api\InvoiceRepositoryInterface;
use Magento\Framework\Api\SearchResultsInterfaceFactory;
use Magento\Framework\Api\Search\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrder;


/**
 * Assign currency exchange rate to orders
 *
 * @author ermanno
 */
class OrderProcessor extends \Magento\Framework\Model\AbstractModel {
    
    const HOME_CURRENCY = 'EUR';

    /**
     * @var Repository
     */
    private $repository;
    
    /**
     * @var Retriever
     */
    private $retriever;
    
    /**
     * @var OrderRepositoryInterface 
     */
    private $orderRepository;
    
    /**
     * @var InvoiceRepositoryInterface 
     */
    private $invoiceRepository;
    
    protected $searchCriteriaBuilder;
    
    public function __construct(
            \Magento\Framework\Model\Context $context, 
            \Magento\Framework\Registry $registry, 
            ExchangeRateRepository $repository, 
            Retriever $retriever, 
            OrderRepositoryInterface $orderRepository, 
            InvoiceRepositoryInterface $invoiceRepository, 
            SearchResultsInterfaceFactory $searchResultsFactory,
            SearchCriteriaBuilder $searchCriteriaBuilder,
            \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null, 
            \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null, 
            array $data = array()) {
        parent::__construct(
                $context, 
                $registry, 
                $resource, 
                $resourceCollection,
                $data);
        $this->repository = $repository;
        $this->retriever = $retriever;
        $this->orderRepository = $orderRepository;
        $this->invoiceRepository = $invoiceRepository;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    public function process($requestedDate)
    {
        $requestedDate = trim($requestedDate);
        $rates = $this->repository->getByDate($requestedDate)->getItems();
        if (empty($rates)) {
            $output->writeln('Retrieving rates, because they were missing.');
            $this->retriever->retrieveDayRates($requestedDate);
            $rates = $this->repository->getByDate($requestedDate)->getItems();
        }
        $orders = $this->getOrders($requestedDate);
        $output->writeln('Found ' . count($orders) . ' to be processed.');
        if (!empty($orders)) {
            foreach ($orders as $order) {
                $this->addCurrencyRate($order, $rates);
            }
        }
    }
    
    public function getInvoices($date = null) {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $searchCriteriaBuilder = $objectManager->create('Magento\Framework\Api\Search\SearchCriteriaBuilder');
        $filterCurrency = $objectManager->create('Magento\Framework\Api\Filter');
        $filterCurrency->setField('order_currency_code')->setValue(self::HOME_CURRENCY)->setConditionType('neq');
        $searchCriteriaBuilder->addFilter($filterCurrency);
        $filterExchangeRate = $objectManager->create('Magento\Framework\Api\Filter');
        $filterExchangeRate->setField('pay_exchangerate')->setValue(0);
        $searchCriteriaBuilder->addFilter($filterExchangeRate);
        if ($date != null) {
            $dateFrom = \DateTime::createFromFormat('Y-m-d', $date);
            $dateTo = clone $dateFrom->setTime(0,0,0);
            $dateTo->add(new \DateInterval('P1D'));
            $filterFrom = $objectManager->create('Magento\Framework\Api\Filter');
            $filterFrom->setField('created_at')->setValue($dateFrom->format('Y-m-d H:i:s'))->setConditionType('gteq');
            $searchCriteriaBuilder->addFilter($filterFrom);
            $filterTo = $objectManager->create('Magento\Framework\Api\Filter');
            $filterTo->setField('created_at')->setValue($dateTo->format('Y-m-d H:i:s'))->setConditionType('lt');
            $searchCriteriaBuilder->addFilter($filterTo);
        }
        $searchCriteria = $searchCriteriaBuilder->addSortOrder('created_at', 'ASC')->create();
        return $this->invoiceRepository->getList($searchCriteria);
    }
    
    /**
     * 
     * @param \Magento\Sales\Model\Order\Invoice $invoice
     * @param type $rates
     * @return boolean
     */
    public function addCurrencyRate($invoice, $rates) {
        $order_currency = $invoice->getOrderCurrencyCode();
        $found_rate = FALSE;
        foreach ($rates as $rate_data) {
            if ($rate_data->getCurrency() == $order_currency) {
                $found_rate = $rate_data;
                break;
            }
        }
        if ($found_rate) {
            $invoice->setPayExchangerate($found_rate->getRate());
            $order = $invoice->getOrder()->setPayExchangerate($found_rate->getRate());
            $this->invoiceRepository->save($invoice);
            $this->orderRepository->save($order);
            return TRUE;
        } else {
            return FALSE;
        }
    }
    
}
